var fs = require('fs'),
    async = require('async');


/**
 * Pads a string to minlength by appending spaces.
 *
 * @param {String} str
 * @param {Number} minlength
 * @return {String}
 * @api public
 */

exports.padRight = function (str, minlength) {
    while (str.length < minlength) {
        str = str + ' ';
    }
    return str;
};


exports.longest = function (arr) {
    return arr.reduce(function (a, x) {
        if (x.length > a) {
            return x.length;
        }
        return a;
    }, 0);
};


/**
 * Reads the version property from Tea's package.json
 */

exports.getTeaVersion = async.memoize(function (callback) {
    exports.readJSON(__dirname + '/../package.json', function (err, pkg) {
        if (err) {
            return callback(err);
        }
        return callback(null, pkg.version);
    });
});


/**
 * Read a file from the filesystem and parse as JSON
 *
 * @param {String} path
 * @param {Function} callback
 * @api public
 */

exports.readJSON = function (path, callback) {
    fs.readFile(path, function (err, content) {
        var val;
        if (err) {
            return callback(err);
        }
        try {
            val = JSON.parse(content.toString());
        }
        catch (e) {
            var stack = e.stack.split('\n').slice(0, 1);
            stack = stack.concat(['\tin ' + path]);
            e.stack = stack.join('\n');
            return callback(e, null);
        }
        callback(null, val);
    });
};


// tests if 'a' is a path below (or equal to) 'b'
exports.isSubPath = function (a, b) {
    var na = path.normalize(a);
    var nb = path.normalize(b);
    var pa = na.split('/');
    var pb = nb.split('/');
    if (pa.length < pb.length) {
        return false;
    }
    for (var i = 0; i < pb.length; i++) {
        if (pa[i] !== pb[i]) {
            return false;
        }
    }
    return true;
};
